'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _postcssValueParser = require('postcss-value-parser');

var _postcssValueParser2 = _interopRequireDefault(_postcssValueParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (functions) {

    function transformValue(value) {
        var promises = [];

        var values = (0, _postcssValueParser2.default)(value).walk(function (node) {
            promises.push(transform(node));
        });

        return Promise.all(promises).then(function () {
            return values.toString();
        });
    }

    function transform(node) {
        if (node.type !== 'function' || !functions.hasOwnProperty(node.value)) return;

        var func = functions[node.value];
        return extractArguments(node.nodes).then(function (args) {
            var invocation = func.apply(func, args);

            var replaceValue = function replaceValue(val) {
                node.type = 'word';
                node.value = val;

                return node;
            };

            if (invocation instanceof Promise) return invocation.then(replaceValue);else return replaceValue(invocation);
        });
    }

    function extractArguments(nodes) {
        return Promise.all(nodes.map(function (node) {
            return node.type === 'function' ? transform(node) : node;
        })).then(function (values) {
            var args = [];
            var last = values.reduce(function (prev, node) {
                if (node.type === 'div' && node.value === ',') {
                    args.push(prev);
                    return '';
                }
                return prev + _postcssValueParser2.default.stringify(node);
            }, '');

            if (last) args.push(last);

            return args;
        });
    }

    return function (node) {
        switch (node.type) {
            case 'decl':
                return transformValue(node.value).then(function (value) {
                    node.value = value;
                });
            case 'atrule':
                return transformValue(node.params).then(function (value) {
                    node.params = value;
                });
            case 'rule':
                return transformValue(node.selector).then(function (value) {
                    node.selector = value;
                });
        }
    };
};

module.exports = exports['default'];